require 'fileutils' unless defined?(FileUtils)
require 'tennpipes-base'
require 'tennpipes-helper'
FileSet.glob_require('tennpipes-memory/{helpers}/*.rb', __FILE__)
require 'moneta'

module Tennpipes
  class << self
    ##
    # Returns the caching engine.
    #
    # @example
    #   # with: Tennpipes.cache = Tennpipes::Cache.new(:File, :dir => /my/cache/path)
    #   Tennpipes.cache['val'] = 'test'
    #   Tennpipes.cache['val'] # => 'test'
    #   Tennpipes.cache.delete('val')
    #   Tennpipes.cache.clear
    #
    def cache
      @_cache
    end

    ##
    # Set the caching engine.
    #
    # @param value
    #   Instance of Moneta store
    #
    # @example
    #   Tennpipes.cache = Tennpipes::Cache.new(:LRUHash) # default choice
    #   Tennpipes.cache = Tennpipes::Cache.new(:File, :dir => Tennpipes.root('tmp', app_name.to_s, 'cache')) # Keeps cached values in file
    #   Tennpipes.cache = Tennpipes::Cache.new(:Memcached) # Uses default server at localhost
    #   Tennpipes.cache = Tennpipes::Cache.new(:Memcached, :server => '127.0.0.1:11211', :exception_retry_limit => 1)
    #   Tennpipes.cache = Tennpipes::Cache.new(:Memcached, :backend => memcached_or_dalli_instance)
    #   Tennpipes.cache = Tennpipes::Cache.new(:Redis) # Uses default server at localhost
    #   Tennpipes.cache = Tennpipes::Cache.new(:Redis, :host => '127.0.0.1', :port => 6379, :db => 0)
    #   Tennpipes.cache = Tennpipes::Cache.new(:Redis, :backend => redis_instance)
    #   Tennpipes.cache = Tennpipes::Cache.new(:Mongo) # Uses default server at localhost
    #   Tennpipes.cache = Tennpipes::Cache.new(:Mongo, :backend => mongo_client_instance)
    #
    #   # You can manage your cache from anywhere in your app:
    #
    #   Tennpipes.cache['val'] = 'test'
    #   Tennpipes.cache['val'] # => 'test'
    #   Tennpipes.cache.delete('val')
    #   Tennpipes.cache.clear
    #
    def cache=(value)
      @_cache= value
    end
  end

  ##
  # This component enables caching of an application's response contents on
  # both page- and fragment-levels. Output cached in this manner is
  # persisted, until it expires or is actively expired, in a configurable store
  # of your choosing. Several common caching stores are supported out of the box.
  #
  module Cache
    class << self
      ##
      # Register these helpers:
      #
      #   Tennpipes::Cache::Helpers::ObjectCache
      #   Tennpipes::Cache::Helpers::CacheStore
      #   Tennpipes::Cache::Helpers::Fragment
      #   Tennpipes::Cache::Helpers::Page
      #
      # for Tennpipes::Application.
      #
      # By default we use FileStore as showed below:
      #
      #   set :cache, Tennpipes::Cache.new(:File, :dir => Tennpipes.root('tmp', app_name.to_s, 'cache'))
      #
      # However, you can also change the file store easily in your app.rb:
      #
      #   set :cache, Tennpipes::Cache.new(:LRUHash) # Keeps cached values in memory
      #   set :cache, Tennpipes::Cache.new(:Memcached) # Uses default server at localhost
      #   set :cache, Tennpipes::Cache.new(:Memcached, '127.0.0.1:11211', :exception_retry_limit => 1)
      #   set :cache, Tennpipes::Cache.new(:Memcached, :backend => memcached_or_dalli_instance)
      #   set :cache, Tennpipes::Cache.new(:Redis) # Uses default server at localhost
      #   set :cache, Tennpipes::Cache.new(:Redis, :host => '127.0.0.1', :port => 6379, :db => 0)
      #   set :cache, Tennpipes::Cache.new(:Redis, :backend => redis_instance)
      #   set :cache, Tennpipes::Cache.new(:Mongo) # Uses default server at localhost
      #   set :cache, Tennpipes::Cache.new(:Mongo, :backend => mongo_client_instance)
      #   set :cache, Tennpipes::Cache.new(:File, :dir => Tennpipes.root('tmp', app_name.to_s, 'cache')) # default choice
      #
      # You can manage your cache from anywhere in your app:
      #
      #   MyApp.cache['val'] = 'test'
      #   MyApp.cache['val'] # => 'test'
      #   MyApp.cache.delete('val')
      #   MyApp.cache.clear
      #
      def registered(app)
        app.helpers Tennpipes::Cache::Helpers::ObjectCache
        app.helpers Tennpipes::Cache::Helpers::CacheStore
        app.helpers Tennpipes::Cache::Helpers::Fragment
        app.helpers Tennpipes::Cache::Helpers::Page
        unless app.respond_to?(:cache)
          cache_dir = Tennpipes.root('tmp', defined?(app.app_name) ? app.app_name.to_s : '', 'cache')
          app.set :cache, Tennpipes::Cache.new(:File, :dir => cache_dir)
        end
        app.disable :caching unless app.respond_to?(:caching)
        included(app)
      end

      def included(base)
        base.extend Tennpipes::Cache::Helpers::Page::ClassMethods
      end

      def tennpipes_route_added(route, verb, path, args, options, block)
        Tennpipes::Cache::Helpers::Page.tennpipes_route_added(route, verb, path, args, options, block)
      end
    end

    def self.new(name, options = {})
      # Activate expiration by default
      options[:expires] = true unless options.include?(:expires)
      Moneta.new(name, options)
    end

    Tennpipes.cache = Tennpipes::Cache.new(:LRUHash)
  end
end
